@@ -28,6 +28,8 @@ import("core.base.hashset")
 import("core.project.rule")
 import("lib.detect.find_tool")
 import("private.utils.batchcmds")
+import("private.utils.rule_groups")
+import("plugins.project.utils.target_cmds", {rootdir = os.programdir()})
 
 -- get minimal cmake version
 function _get_cmake_minver()
@@ -137,7 +139,8 @@ function _sourcebatch_is_built(sourcebatch)
     local rulename = sourcebatch.rulename
     if rulename == "c.build" or rulename == "c++.build"
         or rulename == "asm.build" or rulename == "cuda.build"
-        or rulename == "objc.build" or rulename == "objc++.build" then
+        or rulename == "objc.build" or rulename == "objc++.build"
+        or rulename == "win.sdk.resource" then
         return true
     end
 end
@@ -194,6 +197,55 @@ function _get_flags_from_fileconfig(fileconfig, outputdir, name)
     end
 end
 
+-- get flags from target
+-- @see https://github.com/xmake-io/xmake/issues/3594
+function _get_flags_from_target(target, flagkind)
+    local flags = _get_configs_from_target(target, flagkind)
+    local extraconf = target:extraconf(flagkind)
+    local sourcekind
+    if flagkind == "cflags" then
+        sourcekind = "cc"
+    elseif flagkind == "cxxflags" or flagkind == "cxflags" then
+        sourcekind = "cxx"
+    elseif flagkind == "asflags" then
+        sourcekind = "as"
+    elseif flagkind == "cuflags" then
+        sourcekind = "cu"
+    else
+        raise("unknown flag kind %s", flagkind)
+    end
+    local toolinst = target:compiler(sourcekind)
+
+    -- does this flag belong to this tool?
+    -- @see https://github.com/xmake-io/xmake/issues/3022
+    --
+    -- e.g.
+    -- for all: add_cxxflags("-g")
+    -- only for clang: add_cxxflags("clang::-stdlib=libc++")
+    -- only for clang and multiple flags: add_cxxflags("-stdlib=libc++", "-DFOO", {tools = "clang"})
+    --
+    local result = {}
+    for _, flag in ipairs(flags) do
+        local for_this_tool = true
+        local flagconf = extraconf and extraconf[flag]
+        if type(flag) == "string" and flag:find("::", 1, true) then
+            for_this_tool = false
+            local splitinfo = flag:split("::", {plain = true})
+            local toolname = splitinfo[1]
+            if toolname == toolinst:name() then
+                flag = splitinfo[2]
+                for_this_tool = true
+            end
+        elseif flagconf and flagconf.tools then
+            for_this_tool = table.contains(table.wrap(flagconf.tools), toolinst:name())
+        end
+        if for_this_tool then
+            table.insert(result, flag)
+        end
+    end
+    return result
+end
+
 -- add project info
 function _add_project(cmakelists, languages, outputdir)
 
@@ -537,10 +589,10 @@ end
 
 -- add target compile options
 function _add_target_compile_options(cmakelists, target, outputdir)
-    local cflags   = _get_configs_from_target(target, "cflags")
-    local cxflags  = _get_configs_from_target(target, "cxflags")
-    local cxxflags = _get_configs_from_target(target, "cxxflags")
-    local cuflags  = _get_configs_from_target(target, "cuflags")
+    local cflags   = _get_flags_from_target(target, "cflags")
+    local cxflags  = _get_flags_from_target(target, "cxflags")
+    local cxxflags = _get_flags_from_target(target, "cxxflags")
+    local cuflags  = _get_flags_from_target(target, "cuflags")
     if #cflags > 0 or #cxflags > 0 or #cxxflags > 0 or #cuflags > 0 then
         cmakelists:print("target_compile_options(%s PRIVATE", target:name())
         for _, flag in ipairs(_translate_flags(cflags, outputdir)) do
@@ -893,7 +945,10 @@ function _get_command_string(cmd, outputdir)
 end
 
 -- add target custom commands for batchcmds
-function _add_target_custom_commands_for_batchcmds(cmakelists, target, outputdir, suffix, batchcmds)
+function _add_target_custom_commands_for_batchcmds(cmakelists, target, outputdir, suffix, cmds)
+    if #cmds == 0 then
+        return
+    end
     if suffix == "before" then
         -- ADD_CUSTOM_COMMAND and PRE_BUILD did not work as I expected,
         -- so we need use add_dependencies and fake target to support it.
@@ -902,7 +957,7 @@ function _add_target_custom_commands_for_batchcmds(cmakelists, target, outputdir
         --
         local key = target:name() .. "_" .. hash.uuid():split("-", {plain = true})[1]
         cmakelists:print("add_custom_command(OUTPUT output_%s", key)
-        for _, cmd in ipairs(batchcmds:cmds()) do
+        for _, cmd in ipairs(cmds) do
             local command = _get_command_string(cmd, outputdir)
             if command then
                 cmakelists:print("    COMMAND %s", command)
@@ -914,12 +969,10 @@ function _add_target_custom_commands_for_batchcmds(cmakelists, target, outputdir
         cmakelists:print("    DEPENDS output_%s", key)
         cmakelists:print(")")
         cmakelists:print("add_dependencies(%s target_%s)", target:name(), key)
-    else
+    elseif suffix == "after" then
         cmakelists:print("add_custom_command(TARGET %s", target:name())
-        if suffix == "after" then
-            cmakelists:print("    POST_BUILD")
-        end
-        for _, cmd in ipairs(batchcmds:cmds()) do
+        cmakelists:print("    POST_BUILD")
+        for _, cmd in ipairs(cmds) do
             local command = _get_command_string(cmd, outputdir)
             if command then
                 cmakelists:print("    COMMAND %s", command)
@@ -930,67 +983,26 @@ function _add_target_custom_commands_for_batchcmds(cmakelists, target, outputdir
     end
 end
 
--- add target custom commands for target
-function _add_target_custom_commands_for_target(cmakelists, target, outputdir, suffix)
-    for _, ruleinst in ipairs(target:orderules()) do
-        local scriptname = "buildcmd" .. (suffix and ("_" .. suffix) or "")
-        local script = ruleinst:script(scriptname)
-        if script then
-            local batchcmds_ = batchcmds.new({target = target})
-            script(target, batchcmds_, {})
-            if not batchcmds_:empty() then
-                _add_target_custom_commands_for_batchcmds(cmakelists, target, outputdir, suffix, batchcmds_)
-            end
-        end
-    end
-end
-
--- add target custom commands for object rules
-function _add_target_custom_commands_for_objectrules(cmakelists, target, sourcebatch, outputdir, suffix)
-
-    -- get rule
-    local rulename = assert(sourcebatch.rulename, "unknown rule for sourcebatch!")
-    local ruleinst = assert(target:rule(rulename) or project.rule(rulename) or rule.rule(rulename), "unknown rule: %s", rulename)
-
-    -- generate commands for xx_buildcmd_files
-    local scriptname = "buildcmd_files" .. (suffix and ("_" .. suffix) or "")
-    local script = ruleinst:script(scriptname)
-    if script then
-        local batchcmds_ = batchcmds.new({target = target})
-        script(target, batchcmds_, sourcebatch, {})
-        if not batchcmds_:empty() then
-            _add_target_custom_commands_for_batchcmds(cmakelists, target, outputdir, suffix, batchcmds_)
-        end
-    end
-
-    -- generate commands for xx_buildcmd_file
-    if not script then
-        scriptname = "buildcmd_file" .. (suffix and ("_" .. suffix) or "")
-        script = ruleinst:script(scriptname)
-        if script then
-            local sourcekind = sourcebatch.sourcekind
-            for _, sourcefile in ipairs(sourcebatch.sourcefiles) do
-                local batchcmds_ = batchcmds.new({target = target})
-                script(target, batchcmds_, sourcefile, {})
-                if not batchcmds_:empty() then
-                    _add_target_custom_commands_for_batchcmds(cmakelists, target, outputdir, suffix, batchcmds_)
-                end
-            end
-        end
-    end
-end
-
 -- add target custom commands
 function _add_target_custom_commands(cmakelists, target, outputdir)
-    _add_target_custom_commands_for_target(cmakelists, target, outputdir, "before")
-    for _, sourcebatch in table.orderpairs(target:sourcebatches()) do
-        if not _sourcebatch_is_built(sourcebatch) then
-            _add_target_custom_commands_for_objectrules(cmakelists, target, sourcebatch, outputdir, "before")
-            _add_target_custom_commands_for_objectrules(cmakelists, target, sourcebatch, outputdir)
-            _add_target_custom_commands_for_objectrules(cmakelists, target, sourcebatch, outputdir, "after")
-        end
-    end
-    _add_target_custom_commands_for_target(cmakelists, target, outputdir, "after")
+
+    -- build sourcebatch groups first
+    local sourcegroups = rule_groups.build_sourcebatch_groups(target, target:sourcebatches())
+
+    -- add before commands
+    -- we use irpairs(groups), because the last group that should be given the highest priority.
+    local cmds_before = {}
+    target_cmds.get_target_buildcmd(target, cmds_before, "before")
+    target_cmds.get_target_buildcmd_sourcegroups(target, cmds_before, sourcegroups, "before")
+    -- rule.on_buildcmd_files should also be executed before building the target, as cmake PRE_BUILD does not work.
+    target_cmds.get_target_buildcmd_sourcegroups(target, cmds_before, sourcegroups)
+    _add_target_custom_commands_for_batchcmds(cmakelists, target, outputdir, "before", cmds_before)
+
+    -- add after commands
+    local cmds_after = {}
+    target_cmds.get_target_buildcmd_sourcegroups(target, cmds_after, sourcegroups, "after")
+    target_cmds.get_target_buildcmd(target, cmds_after, "after")
+    _add_target_custom_commands_for_batchcmds(cmakelists, target, outputdir, "after", cmds_after)
 end
 
 -- TODO export target headers (deprecated)
