@@ -316,7 +316,7 @@ function _make_targetinfo(mode, arch, target, vcxprojdir)
         targetinfo.culinkflags = linkinst:linkflags({target = target})
 
         -- save cuda devlink status
-        targetinfo.cudevlink = target:values("cuda.build.devlink")
+        targetinfo.cudevlink = target:policy("build.cuda.devlink") or target:values("cuda.build.devlink")
     end
 
     -- save execution dir (when executed from VS)
@@ -477,8 +477,15 @@ function _make_vsinfo_archs()
             end
             if default_archs then
                 default_archs = hashset.from(table.wrap(default_archs))
-                default_archs:remove("arm64")
-                vsinfo_archs = default_archs:to_array()
+                -- just generate single arch by default to avoid some fails for installing packages.
+                -- @see https://github.com/xmake-io/xmake/issues/3268
+                local arch = config.arch()
+                if default_archs:has(arch) then
+                    vsinfo_archs = { arch }
+                else
+                    default_archs:remove("arm64")
+                    vsinfo_archs = default_archs:to_array()
+                end
             end
         end
     end
@@ -488,68 +495,6 @@ function _make_vsinfo_archs()
     return vsinfo_archs
 end
 
--- config target
-function _config_target(target)
-    for _, rule in ipairs(target:orderules()) do
-        local on_config = rule:script("config")
-        if on_config then
-            on_config(target)
-        end
-    end
-    local on_config = target:script("config")
-    if on_config then
-        on_config(target)
-    end
-end
-
--- config targets
-function _config_targets()
-    for _, target in ipairs(project.ordertargets()) do
-        if target:is_enabled() then
-            _config_target(target)
-        end
-    end
-end
-
--- load rules in the required packages for target
-function _load_package_rules_for_target(target)
-    for _, rulename in ipairs(target:get("rules")) do
-        local packagename = rulename:match("@(.-)/")
-        if packagename then
-            local pkginfo = project.required_package(packagename)
-            if pkginfo then
-                local r = pkginfo:rule(rulename)
-                if r then
-                    target:rule_add(r)
-                    for _, dep in pairs(r:deps()) do
-                        target:rule_add(dep)
-                    end
-                end
-            end
-        end
-    end
-end
-
--- load rules in the required packages for targets
--- @see https://github.com/xmake-io/xmake/issues/2374
---
--- @code
--- add_requires("zlib", {system = false})
--- target("test")
---    set_kind("binary")
---    add_files("src/*.cpp")
---    add_packages("zlib")
---    add_rules("@zlib/test")
--- @endcode
---
-function _load_package_rules_for_targets()
-    for _, target in ipairs(project.ordertargets()) do
-        if target:is_enabled() then
-            _load_package_rules_for_target(target)
-        end
-    end
-end
-
 -- make vstudio project
 function make(outputdir, vsinfo)
 
@@ -598,16 +543,13 @@ function make(outputdir, vsinfo)
                 platform.load(config.plat(), arch):check()
 
                 -- check project options
-                project.check()
+                project.check_options()
 
                 -- install and update requires
                 install_requires()
 
-                -- load package rules for targets
-                _load_package_rules_for_targets()
-
-                -- config targets
-                _config_targets()
+                -- load targets
+                project.load_targets()
 
                 -- update config files
                 generate_configfiles()
